/*
 Copyright (c) 2006 - 2010  Eric J. Feminella  <eric@ericfeminella.com>
 All rights reserved.

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is 
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in 
 all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 THE SOFTWARE.
 
 @internal
 */

package tests.com.ericfeminella.support.signals.binding
{
	import com.ericfeminella.support.signals.binding.SignalDataBinding;
	import tests.com.ericfeminella.support.signals.binding.mocks.MockMessage;
	import tests.com.ericfeminella.support.signals.binding.mocks.MockHost;
	import org.flexunit.Assert;
	
	public final class SignalDataBindingTest
	{
		private static const _hostPropertyName:String = "targetProperty";
		private var _bindings:SignalDataBinding       = null;
		private var _mockMessage:MockMessage          = null;
		private var _host:MockHost                    = null;
		
		[Before]
		public function before() : void
		{
			_bindings    = new SignalDataBinding();
			_mockMessage = new MockMessage();
			_host        = new MockHost();
		}
		
		[Test]
		public function testAddBindableListener() : void
		{
			_bindings.addBindableListener( _mockMessage.messageChanged, _host, _hostPropertyName );
			
			_mockMessage.message = MockMessage.VALID_MESSAGE;
			Assert.assertEquals( _mockMessage.message, _host.targetProperty );
			
			_mockMessage.message = null;
			Assert.assertNull( _host.targetProperty );
			
			_mockMessage.message = MockMessage.INVALID_MESSAGE;
			Assert.assertEquals( _mockMessage.message, _host.targetProperty );
		}
		
		[Test(description="Ensure bindings can be defined multiple times.")]
		public function testAddBindableListenerAllowOverwrites() : void
		{
			_bindings = new SignalDataBinding( false, true );
			_bindings.addBindableListener( _mockMessage.messageChanged, new MockHost(), _hostPropertyName );
			_bindings.addBindableListener( _mockMessage.messageChanged, new MockHost(), _hostPropertyName );
			_bindings.addBindableListener( _mockMessage.messageChanged, new MockHost(), _hostPropertyName );
		}
		
		[Test(error="Error", description="Ensure bindings can only be defined once.")]
		public function testAddBindableListenerSuppressOverwrites() : void
		{
			_bindings = new SignalDataBinding( false, true );
			_bindings.addBindableListener( _mockMessage.messageChanged, new MockHost(), _hostPropertyName );
			_bindings.addBindableListener( _mockMessage.messageChanged, new MockHost(), _hostPropertyName );
			_bindings.addBindableListener( _mockMessage.messageChanged, new MockHost(), _hostPropertyName );
		}
		
		[Test]
		public function testAddOnceBindableListener() : void
		{
			_bindings.addOnceBindableListener( _mockMessage.messageChanged, _host, _hostPropertyName );
			_mockMessage.message = MockMessage.VALID_MESSAGE;
			_mockMessage.message = MockMessage.INVALID_MESSAGE;
			
			Assert.assertEquals( MockMessage.VALID_MESSAGE, _host.targetProperty );
		}
		
		[Test(description="Ensure bindings can be defined multiple times.")]
		public function testAddOnceBindableListenerAllowOverwrites() : void
		{
			_bindings = new SignalDataBinding( false, true );
			_bindings.addBindableListener( _mockMessage.messageChanged, new MockHost(), _hostPropertyName );
			_bindings.addBindableListener( _mockMessage.messageChanged, new MockHost(), _hostPropertyName );
			_bindings.addBindableListener( _mockMessage.messageChanged, new MockHost(), _hostPropertyName );
		}
		
		[Test(error="Error", description="Ensure bindings can only be defined once.")]
		public function testAddOnceBindableListenerSuppressOverwrites() : void
		{
			_bindings = new SignalDataBinding( false, true );
			_bindings.addOnceBindableListener( _mockMessage.messageChanged, new MockHost(), _hostPropertyName );
			_bindings.addOnceBindableListener( _mockMessage.messageChanged, new MockHost(), _hostPropertyName );
			_bindings.addOnceBindableListener( _mockMessage.messageChanged, new MockHost(), _hostPropertyName );
		}
		
		[Test]
		public function testValidHasBindableListener() : void
		{
			_bindings.addBindableListener( _mockMessage.messageChanged, _host, _hostPropertyName );
			
			var actual:Boolean = _bindings.hasBindableListener( new MockHost(), _hostPropertyName );
			Assert.assertTrue( "Expecting confirmation of listener registration", actual );
		}
		
		[Test]
		public function testInvalidHasBindableListener() : void
		{
			var actual:Boolean = _bindings.hasBindableListener( new MockHost(), _hostPropertyName );
			Assert.assertFalse( "Expecting confirmation of non-registered listener", actual );
		}
		
		[After]
		public function after() : void
		{
			_bindings    = null;
			_mockMessage = null;
			_host        = null;
		}
	}
}